def cut_list(list, length):
    """ This function allows to cut a list into parts of a certain length.
        It returns a new list wich takes less memory and contains fo each index a part of the initial list.

        Args :
            list : list of the images path of the whole database\n
            length (int): the length of the parts\n

        Returns :
            list containing the images path cut by parts of <length>

    """

    listing_parts = []
    intervalle_0 = 0
    intervalle_1 = length
    while intervalle_0 <=(len(list)):
        listing_parts.append(list[intervalle_0:intervalle_1])
        intervalle_0 = intervalle_1
        intervalle_1 = intervalle_1 + length
    return listing_parts

def show_face_data(nparray, n=10, title=""):
    """ This function allows to show the faces from a numpy array.

        Args :
            nparray : array containing the images \n
            n (int): the number of images we want to plot (default=10)\n
            title : the title of the plotted faces

        Returns :
            None

    """

    plt.figure(figsize=(30, 5))
    for i in range(n):
        ax = plt.subplot(2,n,i+1)
        plt.imshow(array_to_img(nparray[i]))
        ax.get_xaxis().set_visible(False)
        ax.get_yaxis().set_visible(False)
    plt.suptitle(title, fontsize = 20)
    plt.show()


if __name__=="__main__" :



    import numpy as np
    import matplotlib.pyplot as plt
    from keras.models import Model
    from keras.layers.core import Dense, Dropout, Activation
    from keras.utils import np_utils
    import keras
    import tensorflow as tf
    from keras.preprocessing.image import img_to_array
    from keras.preprocessing.image import array_to_img
    from multiprocessing import Pool
    import os
    from PIL import Image
    from matplotlib import image
    from keras.datasets import mnist
    from keras.layers import Input, Dense, Conv2D, MaxPooling2D, UpSampling2D, Conv2DTranspose, Reshape
    from keras import layers

    # PART 1
    # In this code, we need to work with numpys. To do so, we need to convert our database
    # To an numpy array.
    # UPLOAD THE PATH OF THE DATABASE:
    data_path="../database/img_align_celeba/img_align_celeba"
    listing = os.listdir(data_path)
    #print(listing) #returns a list of all the files of the path
    listarray = [] # creating the array list that will contain the information of our images

    #cut_list was originally here


    # we choose first to work with only 500 images.
    listing_parts=cut_list(listing,500)
    print(len(listing_parts))

    #Once we have uploaded all our images, we resize our images and fit them in numpy array
    from skimage.transform import resize
    for file in listing_parts[0]:
            if file == file + '.DS_Store':
                continue
            chemin= "../database/img_align_celeba/img_align_celeba/" + file
            im = image.imread(chemin)
            resized_img = resize(im,(128,128))
            listarray.append(resized_img)
    print(np.shape(resized_img))
    nparray = np.array(listarray)

    #######################################################################################################

    # PART 2:  THE ENCODER
    #we construct our encoder :
    from sklearn.model_selection import train_test_split
    X_train, X_test = train_test_split(nparray, test_size=0.2, random_state=0)


    input_img = keras.Input(shape=(128, 128, 3))
    # x = layers.Conv2D(8, (3, 3), activation='relu', padding='same')(input_img)
    # x = layers.MaxPooling2D((2, 2), padding='same')(x)
    # x = layers.Conv2D(16, (3, 3), activation='relu', padding='same')(x)
    # x = layers.MaxPooling2D((2, 2), padding='same')(x)
    # x = layers.Conv2D(32, (3, 3), activation='relu', padding='same')(x)
    # x = layers.MaxPooling2D((2, 2), padding='same')(x)
    # x = layers.Conv2D(64, (3, 3), activation='relu', padding='same')(x)
    # x=  layers.MaxPooling2D((2, 2), padding='same')(x)
    # x=  layers.Flatten()(x)
    # encoded=  layers.Dense(1000, activation='relu', name="CODE")(x)
    #
    # ########################################################################################################
    #
    # # PART 3:  THE DECODER
    # x=layers.Dense(4096,activation='relu')(encoded)
    # x=layers.Reshape((8,8,64))(x)
    # x = layers.UpSampling2D((2, 2))(x)
    # x = layers.Conv2D(32, (3, 3), activation='relu', padding='same')(x)
    # x = layers.UpSampling2D((2, 2))(x)
    # x = layers.Conv2D(16, (3, 3), activation='relu', padding='same')(x)
    # x = layers.UpSampling2D((2, 2))(x)
    # x = layers.Conv2D(8, (3, 3), activation='relu', padding='same')(x)
    # x = layers.UpSampling2D((2, 2))(x)

    x = Conv2D(32, (3, 3), activation='relu', padding='same')(input_img)
    x = MaxPooling2D((2, 2))(x)
    x = Conv2D(16, (3, 3), activation='relu', padding='same')(x)
    x = MaxPooling2D((2, 2))(x)
    x = Conv2D(8, (3, 3), activation='relu', padding='same')(x)
    x =  MaxPooling2D((2, 2))(x)
    encoded = layers.Flatten(name="CODE")(x)

    #encoded = layers.Dense(1000, activation='relu', name="CODE")(x)


    #x=layers.Dense(2048,activation='relu')(encoded)
    x = layers.Reshape((16,16,8))(encoded)
    x = Conv2DTranspose(8, (3, 3), activation='relu', padding='same')(x)
    x = UpSampling2D((2, 2))(x)
    x = Conv2DTranspose(16, (3, 3), activation='relu', padding='same')(x)
    x = UpSampling2D((2, 2))(x)
    x = Conv2DTranspose(32, (3, 3), activation='relu', padding='same')(x)
    x = UpSampling2D((2,2))(x)


    decoded = layers.Conv2D(3, (3, 3), activation='sigmoid', padding='same')(x)

    autoencoder = keras.Model(input_img, decoded)
    autoencoder.compile(optimizer='adam', loss='binary_crossentropy', metrics=["accuracy"])
    autoencoder.summary()

    # We train the encoder
    history=autoencoder.fit(X_train, X_train, epochs=150, batch_size=32, shuffle=True, validation_data=(X_test, X_test))

    # We create the decoder model
    Decodeur = Model(encoded, decoded)
    Decodeur.compile(optimizer='adam', loss='mse')
    Decodeur.save("decodeur.h5")

    #######################################################################################################


    #PART 4: THE VECTOR
    # We need now to obtain the encoded vector that will be used for the genetic algorithms part:

    get_encoded_X = Model(inputs=autoencoder.input, outputs=autoencoder.get_layer("CODE").output)
    get_encoded_X.compile(optimizer='adam', loss='mse')
    get_encoded_X.save("encodeur.h5")
    encoded = get_encoded_X.predict(X_test)
    print(len(X_test))
    print(np.shape(encoded))
    #encoded = encoded.reshape(100,100)
    reconstructed = autoencoder.predict(X_test)

    np.save("model/vecteur.npy", encoded) # THE ENCODED VECTOR IS HERE, A npy file is given. to use it for the genetic algorithm
    # you need to reupload it ;)


    #######################################################################################################

    # PART 5: PLOTTING THE PICTURES

    #show_face_data was originally here


    show_face_data(X_test, title="original faces")
    show_face_data(reconstructed, title="reconstructed faces")



    ########################################################################################################

    # PART 6 : PLOTTING MODEL ACCURACY AND loss

    # summarize model for accuracy
    plt.plot(history.history['accuracy'])
    plt.plot(history.history['val_accuracy'])
    plt.title('model accuracy')
    plt.ylabel('accuracy')
    plt.xlabel('epoch')
    plt.legend(['train', 'test'], loc='upper left')
    plt.show()
    # summarize history for loss
    plt.plot(history.history['loss'])
    plt.plot(history.history['val_loss'])
    plt.title('model loss')
    plt.ylabel('loss')
    plt.xlabel('epoch')
    plt.legend(['train', 'test'], loc='upper left')
    plt.show()



#before the initial sample
import numpy as np                   # advanced math library
import matplotlib.pyplot as plt      # plotting routines
import random
#import tensorflow as tf
#from tensorflow import keras
#import h5py
import os
#import cv2
from PIL import Image
import scipy.misc

os.environ['TF_CPP_MIN_LOG_LEVEL'] = '2'

# UPLOAD THE DECODER :
from keras.models import load_model

sample_size=10
##

import doctest
doctest.testmod(verbose=True)


def cross_over(pop, parent, lamb):
    """ This function allows to cross-over the selected parent with random other images with the same characteristics (sex, age and hair/beard wise).
        It returns a new population of mutated vectors while keeping the parent.

        Args :
            pop : encoded images vector of the whole database\n
            parent: the array selected by the user\n
            lamb (int): the size of the total population (children + parent)

        Returns :
            array containing lamb vectors from encoded pictures

        Example :
            >>> len(cross_over(population, population[0], 4))
            4
            >>> population[0] in cross-over(population,population[0], 4)
            True
    """

    n_children = lamb -1
    N = len(pop)
    cross_index = np.random.choice(range(N), n_children)    # sélectionne 3 index au hasard dans notre base de données
    #print(cross_index)
    crossed = [parent]
    for i in cross_index:
        child=[]
        for j in range (len(parent)):
            child.append(np.average([parent[j],pop[i][j]], weights=[0.4,0.6])) # on fait la moyenne pour chaque attribut entre le vecteur parent et le vecteur choisi aléatoirement
        crossed.append(child)
    return np.asarray(crossed)


def mutation(pop):
    """ This function allows to mutate the picture's attributes using Gaussian distribution.
        It returns a new population of mutated vectors.

        Args :
            pop : encoded images vector to mute

        Returns :
            nparray containing modified vectors from encoded pictures

    """
    std=pop.std(axis=0)
    N = len(pop)
    for i in range(1,len(pop)):
        random_value=np.random.normal(0,1)  #pour chaque enfant on choisi alpha
        for j in range(1,len(pop[i])):
            pop[i][j]+=random_value*std[i]
    return pop


def get_children_from_parent(pop, parent, lamb):
    """ This function allows to cross-over the parent pictures with other pictures and mutate the result picture to add diversity.
        It returns a new population of mutated vectors.

        Args :
            pop : encoded images vector of the whole database\n
            parent: the array selected by the user\n
            lamb (int): the size of the total population (children + parent)

        Returns :
            array containing lamb vectors from encoded pictures
    """
    children=cross_over(pop, parent, lamb)
    mutated_children=mutation(children)
    return mutated_children




if __name__=="__main__":



    #import doctest
    #doctest.testmod(verbose=True)


    decoder = load_model("decodeur.h5")
    encoded_imgs=np.load("img_female_old_straight.csv.npy")



## Crossing cross_over
    children=cross_over(encoded_imgs, encoded_imgs[50], 4)
    children_decoded = decoder.predict(children)
    for i in range (len(children)):
        ax = plt.subplot(1, len(children), i + 1 )
        plt.imshow(children_decoded[i].reshape(128,128,3))
    plt.show()


## Mutation

    mutated_children=mutation(children)
    children_decoded2 = decoder.predict(mutated_children)
    for i in range (len(children)):
        ax = plt.subplot(1, len(children), i + 1 )
        plt.imshow(children_decoded2[i].reshape(128,128,3))
        plt.title("Mutated")
    plt.show()

## Generate new population

    x=get_children_from_parent(encoded_imgs, encoded_imgs[134], 4)
    decoded_x=decoder.predict(x)
    for i in range (len(x)):
        ax = plt.subplot(1, len(x), i + 1 )
        plt.imshow(decoded_x[i].reshape(128,128,3))
    plt.show()



import pandas as pds
import numpy as np
import csv
from matplotlib import image
from skimage.transform import resize
import os

def filter(path):
    """
    This function creates 8 files containing different pictures sorted according specific attributes.

    Args :
        path : path to the database

    Returns :
        None

    """

    db = pds.read_csv(path, sep=",") # load a pandas dataframe from csv in current directory
    #print(db.head())
    #print(db.shape)

    #sorting male/female
    db_male=db.loc[db['Male']==1]
    db_female=db.loc[db['Male']==-1]


    #sorting young/old

    db_male_young=db_male.loc[db['Young']==1]
    db_male_old=db_male.loc[db['Young']==-1]
    #print("Liste des hommes jeunes :",db_male_young)

    db_female_young=db_female.loc[db['Young']==1]
    db_female_old=db_female.loc[db['Young']==-1]
    #print("Liste des femmes jeunes :", db_female_young)

    #sorting beard/no beard

    db_male_young_nobeard=db_male_young.loc[db['No_Beard']==1]
    # Enregister notre structure de données dans un fichier.csv
    db_male_young_nobeard.to_csv('male_young_nobeard.csv',mode='w+')
    db_male_young_beard=db_male_young.loc[db['No_Beard']==-1]
    db_male_young_beard.to_csv('male_young_beard.csv',mode='w+')

    db_male_old_nobeard=db_male_old.loc[db['No_Beard']==1]
    db_male_old_nobeard.to_csv('male_old_nobeard.csv',mode='w+')
    db_male_old_beard=db_male_old.loc[db['No_Beard']==-1]
    db_male_old_beard.to_csv('male_old_beard.csv',mode='w+')

    #print("Liste des hommes jeunes avec barbe :", db_male_young_beard)
    #print("Liste des hommes vieux sans barbe :", db_male_old_nobeard)

    #sorting wavy/straight hair

    db_female_young_wavy=db_female_young.loc[db['Straight_Hair']==-1]
    db_female_young_wavy.to_csv('female_young_wavy.csv',mode='w+')
    db_female_young_straight=db_female_young.loc[db['Straight_Hair']==1]
    db_female_young_straight.to_csv('female_young_straight.csv',mode='w+')

    db_female_old_wavy=db_female_old.loc[db['Straight_Hair']==-1]
    db_female_old_wavy.to_csv('female_old_wavy.csv',mode='w+')
    db_female_old_straight=db_female_old.loc[db['Straight_Hair']==1]
    db_female_old_straight.to_csv('female_old_straight.csv',mode='w+')

    #print("Liste des femmes jeunes avec straight :", db_female_young_straight)
    #print("Liste des femmes jeunes sans straight :", db_female_young_wavy)
    #print("Liste des femmes vieilles sans straight :", db_female_old_wavy)


def split_encoded_images(encoder, csv_file):

    """
    This function creates a file containing encoded vectors of images with the same caracteristics.

    Args :
        encoder : encoder loaded from file "encodeur.h5"
        csv_file : path to the csv file containing the listed pictures with the same caracteristics

    Returns :
        None

    """

    f=open(csv_file)
    csv_f = csv.reader(f)
    list_ref=[]

    for row in csv_f:
        ref=row[1]
        #print(ref)
        list_ref.append(ref)

    original_images=[]
    data_path="../database/img_align_celeba/img_align_celeba/"
    for file in list_ref[1:1001]:
            chemin= data_path+file
            im = image.imread(chemin)
            resized_img = resize(im,(128,128))
            original_images.append(resized_img)
    #print(np.shape(resized_img))
    nparray = np.array(original_images)

    #encode images and save them in a file :
    encoded_images = encoder.predict(nparray)
    np.save("images/img_"+str(csv_file[10:])+".npy", encoded_images)


#modif

if __name__=="__main__" :
    # UPLOAD THE DECODER :
    from keras.models import load_model
    os.environ['TF_CPP_MIN_LOG_LEVEL'] = '2'
    #decodeur=np.save(".../decodeur.h5",x[0])
    encoder = load_model("model/encodeur.h5")

    #filter('databases/list_attr_celeba.csv')
    #db = pds.read_csv('female_young_wavy.csv', sep=",")
    #print(db["image_id"][0])




    split_encoded_images(encoder, "databases/female_old_wavy.csv")
    vecteur=np.load("images/img_female_old_wavy.csv.npy")
    print(len(vecteur))
    print(len(vecteur[0]))



import tkinter
import numpy as np
import pandas as pds
import matplotlib.pyplot as plt
from PIL import ImageTk, Image
import random
import evolutionary
import os




### CHOOSE CHARACTERISTICS
characteristics={"woman":False,"man":False,"young":False,"old":False,"beard":False,"no_beard":False,"straight":False,"no_straight":False}
#print(characteristics )


def initialize() :
    """ This function allows to put all the agressor's characteristics to 'False'.

    """
    for cle, valeur in characteristics.items() :
        characteristics[cle]=False
    #print(characteristics)


def onClick(event):

    """ This function allows to generate a new interface to ask and store the characteristics of the agressor.

    """

    for c in myWindow.winfo_children():
        c.destroy()
    initialize()
    mainFrame=tkinter.Frame(myWindow,bg='LightSkyBlue3',width=30, borderwidth=3, relief='groove')
    mainFrame.pack(side='top', padx=30, pady=30,fill="x" )
    tkinter.Label(mainFrame,bg='LightSkyBlue3',text="What did the agressor look like ?",font=(10)).pack(padx=20,pady=20)

    #the women/man frame
    Frame1=tkinter.Frame(myWindow,borderwidth=3, relief='groove')
    Frame1.pack(padx=5, pady=5)

    #to select only one option
    value = tkinter.StringVar()
    fButton=tkinter.Radiobutton(Frame1,text="Woman",variable=value, value=1)
    fButton.pack(side='left', padx=5, pady=5)
    fButton.bind('<Button-1>', fevent)

    hButton=tkinter.Radiobutton(Frame1,text="Man",variable=value, value=2)
    hButton.pack(side='right', padx=5, pady=5)
    hButton.bind('<Button-1>', hevent)

    #the young/old frame
    Frame2=tkinter.Frame(myWindow,borderwidth=3, relief='groove')
    Frame2.pack(padx=5, pady=5)

    value = tkinter.StringVar()
    yButton=tkinter.Radiobutton(Frame2,text="Young",variable=value, value=1)
    yButton.pack(side='left', padx=5, pady=5)
    yButton.bind('<Button-1>', yevent)

    oButton=tkinter.Radiobutton(Frame2,text="Old",variable=value, value=2)
    oButton.pack(side='right', padx=5, pady=5)
    oButton.bind('<Button-1>', oevent)


    #show a new window to propose some pictures depending on the characteristics
    myButton=tkinter.Button(myWindow,text='Next', width=50, bg="LightSkyBlue4", font=(1))
    myButton.pack(padx=20, pady=20, fill="x")
    myButton.bind('<ButtonRelease-1>',sex_characteristics)

def fevent(event):
    """This function is for choosing the gender which is 'woman' not 'man'.
    """

    characteristics["woman"]=True
    characteristics["man"]=False

def hevent(event):
    """This function is for choosing the gender which is 'man' not 'woman'.
    """
    characteristics["man"]=True
    characteristics["woman"]=False

def yevent(event):
    """This function is for choosing the age group which is 'young' not 'old'.
    """
    characteristics["young"]=True
    characteristics["old"]=False

def oevent(event):
    """This function is for choosing the age group which is 'old' not 'young'.
    """
    characteristics["old"]=True
    characteristics["young"]=False

def bevent(event):
    """This function is for choosing the characteristic for man which is 'beard' not 'no beard'.
    """
    characteristics["beard"]=True
    characteristics["no_beard"]=False

def nbevent(event):
    """This function is for choosing the characteristic for man which is 'no beard' not 'beard'.
    """
    characteristics["no_beard"]=True
    characteristics["beard"]=False

def sevent(event):
    """This function is for choosing the characteristic for woman which is 'straight hair' not 'wavy hair'.
    """
    characteristics["straight"]=True
    characteristics["no_straight"]=False

def nsevent(event):
    """This function is for choosing the characteristic for woman which is 'wavy hair' not 'straight hair'.
    """
    characteristics["no_straight"]=True
    characteristics["straight"]=False

def sex_characteristics(event):
    """This function is for asking the characteristic for each gender.
       'beard' or 'no beard' for man and 'straight hair' or 'wavy hair' for woman.
    """
    if characteristics["man"]==True:
        mainFrame=tkinter.Frame(myWindow,bg='LightSkyBlue3',width=30, borderwidth=3, relief='groove')
        mainFrame.pack(side='top', padx=30, pady=30,fill="x" )
        tkinter.Label(mainFrame,bg='LightSkyBlue3',text='And his beard?',font=(10)).pack(padx=20,pady=20)

        #the beard/no beard frame
        Frame3=tkinter.Frame(myWindow,borderwidth=3, relief='groove')
        Frame3.pack(padx=5, pady=5)

        value = tkinter.StringVar()
        bButton=tkinter.Radiobutton(Frame3,text="Beard",variable=value, value=1)
        bButton.pack(side='left', padx=5, pady=5)
        bButton.bind('<Button-1>', bevent)

        nbButton=tkinter.Radiobutton(Frame3,text="No Beard",variable=value, value=2)
        nbButton.pack(side='right', padx=5, pady=5)
        nbButton.bind('<Button-1>', nbevent)

    elif characteristics["man"]==False:
        mainFrame=tkinter.Frame(myWindow,bg='LightSkyBlue3',width=30, borderwidth=3, relief='groove')
        mainFrame.pack(side='top', padx=30, pady=30,fill="x" )
        tkinter.Label(mainFrame,bg='LightSkyBlue3',text='And her hair?',font=(10)).pack(padx=20,pady=20)

        #the straight/no straight hair frame
        Frame4=tkinter.Frame(myWindow,borderwidth=3, relief='groove')
        Frame4.pack(padx=5, pady=5)

        value = tkinter.StringVar()
        sButton=tkinter.Radiobutton(Frame4,text="Straight Hair",variable=value, value=1)
        sButton.pack(side='left', padx=5, pady=5)
        sButton.bind('<Button-1>', sevent)

        nsButton=tkinter.Radiobutton(Frame4,text="Wavy Hair",variable=value, value=2)
        nsButton.pack(side='right', padx=5, pady=5)
        nsButton.bind('<Button-1>', nsevent)

    myButton=tkinter.Button(myWindow,text='Show portraits', width=50, bg="LightSkyBlue4", font=(1))
    myButton.pack(padx=20, pady=20, fill="x")
    myButton.bind('<ButtonRelease-1>',inital_population)

#base de données correspondant aux critères
def choice_database(char):
    """This function is for returning the images according to the user's selection.

        Args :
            char : characteristic for the aggressor.

        Returns :
            returns the vector according to the user's selection.
    """
    if char["woman"] and char["young"] and char["straight"]:
        return 'img_female_young_straight.csv.npy'
    if char["woman"] and char["young"] and char["no_straight"]:
        return 'img_female_young_wavy.csv.npy'
    if char["woman"] and char["old"] and char["straight"]:
        return 'img_female_old_straight.csv.npy'
    if char["woman"] and char["old"] and char["no_straight"]:
        return 'img_female_old_wavy.csv.npy'
    if char["man"] and char["young"] and char["beard"]:
        return 'img_male_young_beard.csv.npy'
    if char["man"] and char["young"] and char["no_beard"]:
        return 'img_male_young_nobeard.csv.npy'
    if char["man"] and char["old"] and char["beard"]:
        return 'img_male_old_beard.csv.npy'
    if char["man"] and char["old"] and char["no_beard"]:
        return 'img_male_old_nobeard.csv.npy'
    else :
        print('Im in else')
        return None


def reselect(event) :
    """This function is about asking the user for selecting the gender or characteristic when he/she did not clicked the button.
    """
    for c in myWindow.winfo_children():
        c.destroy()
    mainFrame=tkinter.Frame(myWindow,bg='LightSkyBlue3',width=30, borderwidth=3, relief='groove')
    mainFrame.pack(side='top', padx=30, pady=30,fill="x" )
    tkinter.Label(mainFrame,bg='LightSkyBlue3',text='A data is missing, please reselect',font=(10)).pack(padx=40,pady=40,fill="x")
    myButton=tkinter.Button(myWindow,text='Reselect', width=50, bg="LightSkyBlue4",font=(10))
    myButton.place(relx=0.5, rely=0.5, anchor="center")
    myButton.bind('<ButtonRelease-1>',onClick)


##PRESENTATION DE LA POPULATION INITIAL

def inital_population(event):
    """This function is for showing the initial 10 pictures of the suspects according to the user's choice.
    """
    database=choice_database(characteristics)
    if database == None:
        print('Im in none')
        reselect(event)
    if database != None:
        for c in myWindow.winfo_children():
            c.destroy()
        #print(characteristics)
        tkinter.Label(myWindow,bg='LightSkyBlue3',text='Do you recognise one of the suspects ?',font=(10)).pack(padx=10,pady=10,fill="x")
        #place(relx=0.5, rely=0.1, anchor="center",fill="x")
        #.pack(padx=10,pady=10)

        encoded_imgs=np.load(database, allow_pickle=True)
        sample_size=10
        img_index = np.random.choice(range(len(encoded_imgs)), sample_size)
        pop0=encoded_imgs[img_index]
        #print(len(pop0[1]))

        pop0_decoded_imgs = decoder.predict(pop0)

        photo=[]
        for i in range (len(pop0_decoded_imgs)) :

            plt.imsave("img", pop0_decoded_imgs[i].reshape(128,128,3), format="png")
            photo.append(ImageTk.PhotoImage(Image.open("img")))

    #for i in range(5):
    #    myWindow.columnconfigure(i, weight=1)

    #myWindow.rowconfigure(1, weight=1)


        imLab1=tkinter.Label(myWindow,image=photo[0])
        imLab1.place(relx=0.10, rely=0.35, anchor="center")
        imLab1.bind('<Button-1>', lambda event,  pop=encoded_imgs, parent=pop0[0], nb_children=4: chooseimage(pop, parent, nb_children))

        imLab2=tkinter.Label(myWindow,image=photo[1])
        imLab2.place(relx=0.30, rely=0.35, anchor="center")
        imLab2.bind('<Button-1>', lambda event,  pop=encoded_imgs, parent=pop0[1], nb_children=4: chooseimage(pop, parent, nb_children))

        imLab3=tkinter.Label(myWindow,image=photo[2])
        imLab3.place(relx=0.5, rely=0.35, anchor="center")
        imLab3.bind('<Button-1>', lambda event,  pop=encoded_imgs, parent=pop0[2], nb_children=4: chooseimage(pop, parent, nb_children))

        imLab4=tkinter.Label(myWindow,image=photo[3])
        imLab4.place(relx=0.70, rely=0.35, anchor="center")
        imLab4.bind('<Button-1>', lambda event,  pop=encoded_imgs, parent=pop0[3], nb_children=4: chooseimage(pop, parent, nb_children))

        imLab5=tkinter.Label(myWindow,image=photo[4])
        imLab5.place(relx=0.9, rely=0.35, anchor="center")
        imLab5.bind('<Button-1>', lambda event,  pop=encoded_imgs, parent=pop0[4], nb_children=4: chooseimage(pop, parent, nb_children))

        imLab6=tkinter.Label(myWindow,image=photo[5])
        imLab6.place(relx=0.10, rely=0.65, anchor="center")
        imLab6.bind('<Button-1>', lambda event,  pop=encoded_imgs, parent=pop0[5], nb_children=4: chooseimage(pop, parent, nb_children))

        imLab7=tkinter.Label(myWindow,image=photo[6])
        imLab7.place(relx=0.30, rely=0.65, anchor="center")
        imLab7.bind('<Button-1>', lambda event,  pop=encoded_imgs, parent=pop0[6], nb_children=4: chooseimage(pop, parent, nb_children))

        imLab8=tkinter.Label(myWindow,image=photo[7])
        imLab8.place(relx=0.5, rely=0.65, anchor="center")
        imLab8.bind('<Button-1>', lambda event,  pop=encoded_imgs, parent=pop0[7], nb_children=4: chooseimage(pop, parent, nb_children))

        imLab9=tkinter.Label(myWindow,image=photo[8])
        imLab9.place(relx=0.70, rely=0.65, anchor="center")
        imLab9.bind('<Button-1>', lambda event,  pop=encoded_imgs, parent=pop0[8], nb_children=4: chooseimage(pop, parent, nb_children))

        imLab10=tkinter.Label(myWindow,image=photo[9])
        imLab10.place(relx=0.90, rely=0.65, anchor="center")
        imLab10.bind('<Button-1>', lambda event,  pop=encoded_imgs, parent=pop0[9], nb_children=4: chooseimage(pop, parent, nb_children))


        myWindow.mainloop()




###### PRESENT CHILDREN

def chooseimage(pop, parent, nb_children) :
    """This function generates a new interface for asking the user whether he/she recognizes the suspects.

        Args :
            pop : array of the encoded images\n
            parent : selected array of the encoded image chosen by the user\n
            nb_children(int) : the number of children generated from the parent\n

        Returns :
            None
    """
    for c in myWindow.winfo_children():
        c.destroy()
    tkinter.Label(myWindow,bg='LightSkyBlue3',text='Do you recognise one of the suspects ?',font=(10)).pack(padx=10,pady=10,fill="x")
    #place(relx=0.5, rely=0.1, anchor="center",fill="x")
    #print(i)
    new_pop=evolutionary.get_children_from_parent(pop, parent, nb_children)
    children_decoded_imgs = decoder.predict(new_pop)
    #print(len(children_decoded_imgs))
    photo=[]
    for j in range (len(children_decoded_imgs)) :
        plt.imsave("img", children_decoded_imgs[j].reshape(128,128,3), format="png")
        photo.append(ImageTk.PhotoImage(Image.open("img")))

    #print(len(photo))
    for k in range(3):
        myWindow.columnconfigure(k, weight=1)

    #myWindow.rowconfigure(1, weight=1)

    imLab1=tkinter.Label(myWindow,image=photo[0])
    imLab1.place(relx=0.5, rely=0.35, anchor="center")
    imLab1.bind('<Button-1>', lambda event, photo=photo[0], pop=pop, parent=new_pop[0], nb_children=4: end_or_continue(photo, pop, parent, nb_children))

    imLab2=tkinter.Label(myWindow,image=photo[1])
    imLab2.place(relx=0.3, rely=0.65, anchor="center")
    imLab2.bind('<Button-1>', lambda event, photo=photo[1], pop=pop, parent=new_pop[1], nb_children=4: end_or_continue(photo, pop, parent, nb_children))

    imLab3=tkinter.Label(myWindow,image=photo[2])
    imLab3.place(relx=0.5, rely=0.65, anchor="center")
    imLab3.bind('<Button-1>', lambda event, photo=photo[2], pop=pop, parent=new_pop[2], nb_children=4: end_or_continue(photo, pop, parent, nb_children))

    imLab4=tkinter.Label(myWindow,image=photo[3])
    imLab4.place(relx=0.7, rely=0.65, anchor="center")
    imLab4.bind('<Button-1>', lambda event, photo=photo[3], pop=pop, parent=new_pop[3], nb_children=4: end_or_continue(photo, pop, parent, nb_children))

    pause=tkinter.Button(myWindow,text='Need a break ?', width=30, bg='LightSkyBlue4', font=(1))
    pause.place(relx=0.5, rely=0.9, anchor="center")
    pause.bind('<Button-1>', lambda event, pop=pop, parent=parent, nb_children=4: break_time(pop, parent, nb_children))

    myWindow.mainloop()

def break_time(pop, parent, nb_children):
    """This function offers the user a break time with a peaceful picture wtih cute Koala.
       The user can have the break time whenever he/she wants it.

       Args:
            pop : array of the encoded images\n
            parent : selected array of the encoded image chosen by the user\n
            nb_children(int) : the number of children generated from the parent\n

       Returns:
            None
    """

    for c in myWindow.winfo_children():
        c.destroy()
    Frame=tkinter.Frame(myWindow,borderwidth=3, relief='groove')
    Frame.pack(side='top', padx=10, pady=20,expand="yes",fill="both")

        #place(relx=0.5, rely=0.1, anchor="center",fill='x')

    global impause
    impause = ImageTk.PhotoImage(file = "reposant.jpg")
    tkinter.Label(Frame,image=impause).pack()
    continuebutton=tkinter.Button(Frame,text='Continue', width=30, bg='LightSkyBlue4', font=(1))
    continuebutton.place(relx=0.5, rely=0.85, anchor="center")
    continuebutton.bind('<ButtonRelease-1>', lambda event, pop=pop, parent=parent, nb_children=4 : chooseimage(pop, parent, nb_children))

### End loop or continue

def end_or_continue(photo, pop, parent, nb_children):
    """This function generates a new interface for asking the user whether he/she found the suspect.
        It also asks the user if he/she wants other pictures of suspects.

        Arg :
            photo : photo of the selected parent
            pop : array of the encoded images\n
            parent : selected array of the encoded image chosen by the user\n
            nb_children(int) : the number of children generated from the parent\n

        Returns :
            None
    """

    for c in myWindow.winfo_children():
        c.destroy()
    tkinter.Label(myWindow,bg='LightSkyBlue3',text='Is this your agressor ?',font=(10)).pack(padx=10,pady=10,fill="x")
    #place(relx=0.5, rely=0.1, anchor="center",fill='x')

    imLab1=tkinter.Label(myWindow,image=photo)
    imLab1.place(relx=.5,rely=.5, anchor="center")


    myEndButton=tkinter.Button(myWindow,text='I have found my suspect', width=30, bg='LightSkyBlue4', font=(1))
    myEndButton.place(relx=0.3, rely=0.8, anchor="center")
    myEndButton.bind('<ButtonRelease-1>',lambda event, photo=photo: found_agressor(photo))

    myContinueButton=tkinter.Button(myWindow,text='Show other portraits', width=30, bg='LightSkyBlue4', font=(1))
    myContinueButton.place(relx=0.7, rely=0.8, anchor="center")
    myContinueButton.bind('<ButtonRelease-1>',lambda event, pop=pop, parent=parent, nb_children=4 : chooseimage(pop, parent, nb_children))




### CRIMINAL FOUND

def found_agressor(photo):
    """ This function allows to generate the final interface showing the robot portrait of the agressor.

    Args :
        photo : the photo of the agressor\n

    Returns :
        None

    """
    for c in myWindow.winfo_children():
        c.destroy()
    Frame=tkinter.Frame(myWindow,borderwidth=3, relief='groove')
    Frame.pack(side='top', padx=10, pady=20,expand="yes",fill="both")

    #place(relx=0.5, rely=0.1, anchor="center",fill='x')

    imLab1=tkinter.Label(Frame,image=photo)
    imLab1.place(relx=.5,rely=.6, anchor="center")

    Frame1=tkinter.Frame(Frame,bg='LightSkyBlue3',borderwidth=1, relief='groove')
    Frame1.pack( padx=10, pady=10,fill="x")
    tkinter.Label(Frame1,bg='LightSkyBlue3',text='Portrait of the suspect',font=(20)).pack(padx=10,pady=10,fill="x")

    tkinter.Label(Frame,text='Thank you for your collaboration !',font=(10)).pack(padx=10,pady=10,fill="x")
    global emoji
    emoji = ImageTk.PhotoImage(file = "detective.png")
    tkinter.Label(Frame,image=emoji).pack()
    #place(relx=0.5, rely=0.9, anchor="center",fill='x')



if __name__=="__main__" :

    from keras.models import load_model
    os.environ['TF_CPP_MIN_LOG_LEVEL'] = '2'

    decoder = load_model("decodeur.h5")


    #First Window

    myWindow=tkinter.Tk()
    myWindow.geometry("1000x700")
    myWindow['bg']='white'




    Frame=tkinter.Frame(myWindow,borderwidth=3, relief='groove')
    Frame.pack(side='top', padx=10, pady=20,expand="yes",fill="both")
    logo = ImageTk.PhotoImage(file = "logo_insa.png")
    tkinter.Label(Frame,image=logo).pack()

    title= tkinter.Label(Frame,text='Face Hunters',font=(20))
    title.place(relx=0.5, rely=0.35, anchor="center")

    descLabel=tkinter.Label(Frame, text='This is an application developped by 4BIM INSA students to help you create a robot portrait of your agressor. \n We will show you several faces, you just have to click on the photo to select it !  ')
    descLabel.place(relx=0.5, rely=0.5, anchor="center")
    #descLabel.pack()

    emoji1 = ImageTk.PhotoImage(file = "detective_fem.jpg")
    tkinter.Label(Frame,image=emoji1).place(relx=0.4, rely=0.7, anchor="center")
    emoji2 = ImageTk.PhotoImage(file = "detective_hom.png")
    tkinter.Label(Frame,image=emoji2).place(relx=0.6, rely=0.7, anchor="center")

    myButton=tkinter.Button(Frame,text='Start !', font=(10), width=50, bg="LightSteelBlue4")
    myButton.place(relx=0.5, rely=0.9, anchor="center")
    myButton.bind('<ButtonRelease-1>',onClick)


    myWindow.mainloop()



from setuptools import setup
from setuptools import find_packages

setup(
  name = 'face_hunters',         # How you named your package folder (MyLib)
  packages = find_packages(),   # Chose the same as "name"
  version = '0.6.5',      # Start with a small number and increase it with every change you make
  license='MIT',        # Chose a license from here: https://help.github.com/articles/licensing-a-repository
  description = 'This is an application developped by 4BIM INSA students to help you create a robot portait of your agressor.',   # Give a short description about your library
  author = 'G3',                   # Type in your name
  author_email = 'lucieschw@gmail.com',      # Type in your E-Mail
  url = 'https://github.com/lucieschwoertzig/face_hunters',   # Provide either the link to your github or to your website
  #download_url = 'https://github.com/lucieschwoertzig/face_hunters/archive/refs/tags/v_4.tar.gz',    # I explain this later on
  keywords = ['PORTRAITS', 'AUTOENCODER', 'GENETIC'],   # Keywords that define your package best
  install_requires=[            # I get to this in a second
          'numpy',
          'pandas',
          'pillow',
          'keras',
          'tensorflow',
          'matplotlib',
          'scikit-image',
          'sklearn',
          'scipy'
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',      # Define that your audience are developers
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   # Again, pick a license
    'Programming Language :: Python :: 3',      #Specify which pyhton versions that you want to support
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
  ],
  package_data={'face_hunters':['*.png', '*.jpg', '*.h5', '*.npy'], 'build':['doctrees/*.doctree', 'doctrees/*.pickle', 'html/*.html', 'html/*.js', 'html/*.inv']},
)
